(module chain (lib "slideshow.ss" "slideshow")
  (require (lib "mred.ss" "mred")
           (lib "class.ss")
           (lib "list.ss")
           (lib "code.ss" "slideshow")
           (lib "balloon.ss" "texpict")
           (lib "flash.ss" "texpict")
           (lib "unit.ss")
           (only (lib "math.ss") pi)
           "arrow.ss")
  
  (provide (all-defined))
  
  ;; Grep for "=>" for animation-implementation examples

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; Animation player
  
  ;; Create a slide sequence where `mid' takes a number from 0.0 to 1.0.
  ;; The 0.0 slide will wit until you advance, but the remaining ones will
  ;; time out automatically to create the animation.
  (define (play title mid)
    (if title
        (slide/title/center title (mid 0))
        (slide/center (mid 0)))
    (map (lambda (n)
           (slide/title/center/timeout title 0.05 (mid n)))
         (let ([cnt 10])
           (let loop ([n cnt])
             (if (zero? n)
                 null
                 (cons (/ (- cnt -1 n) 1.0 cnt)
                       (loop (sub1 n))))))))
  
  ;; Create a sequences of N `play' sequences, where `mid' takes
  ;; N arguments, each a number between 0.0 and 1.0. Initially, all
  ;; arguments will be 0.0. The first argument goes from 0.0 to 1.0
  ;; for the first `play' sequence, and then it stays at 1.0 while
  ;; the second goes from 0.0 to 1.0 for the second sequence, etc.
  (define (play-n title mid)
    (let ([n (procedure-arity mid)])
      (let loop ([post (vector->list (make-vector n))]
                 [pre null])
        (if (null? post)
            (slide/title/center title (apply mid pre))
            (begin
              (play title
                    (lambda (n)
                      (apply mid (append pre (list n) (cdr post)))))
              (loop (cdr post) (cons 1.0 pre)))))))


  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; Animation combinators
  
  ;; "Morph" from one pict to another. Use `combine' to align
  ;; the picts relative to another. Only the bounding box is
  ;; actually morphed; the drawing part transitions by fading
  ;; the original `a' out and the new `b' in. The `n' argument
  ;; ranges from 0.0 (= `a') to 1.0 (= `b').
  (define (fade-pict* combine n a b)
    ;; Combine ghosts of scaled pictures:
    (let ([orig (combine (cellophane a (- 1.0 n))
                         (cellophane b n))])
      (cond
        [(zero? n) (refocus orig a)]
        [(= n 1.0) (refocus orig b)]
        [else
         (let-values ([(atx aty) (ltl-find orig a)]
                      [(abx aby) (rbl-find orig a)]
                      [(btx bty) (ltl-find orig b)]
                      [(bbx bby) (rbl-find orig b)])
           (let ([da (+ aty (* (- bty aty) n))]
                 [dd (- (pict-height orig)
                        (+ aby (* (- bby aby) n)))])
             (let ([p (make-pict (pict-draw orig)
                                 (pict-width orig)
                                 (pict-height orig)
                                 da
                                 dd
                                 (list (make-child orig 0 0 1 1))
                                 #f)])
               (let ([left (+ atx (* (- btx atx) n))]
                     [right (+ abx (* (- bbx abx) n))])
                 (let ([hp (inset p
                                  (- left)
                                  0
                                  (- right (pict-width p))
                                  0)])
                   (let-values ([(atx aty) (lt-find hp a)]
                                [(abx aby) (lb-find hp a)]
                                [(btx bty) (lt-find hp b)]
                                [(bbx bby) (lb-find hp b)])
                     (let ([top (+ aty (* (- bty aty) n))]
                           [bottom (+ aby (* (- bby aby) n))])
                       (inset hp
                              0
                              (- top)
                              0
                              (- bottom (pict-height hp))))))))))])))
                             
  ;; Usually, combine using cc-superimpose:
  (define (fade-pict n a b)
    (fade-pict* cc-superimpose n a b))
  
  ;; Pin `p' into `base', sliding from `p-from' to `p-to'
  ;;  (which are picts within `base') as `n' goes from 0.0 to 1.0.
  ;; The `p-from' and `p-to' picts are typically ghosts of
  ;;   `p' within `base', but they can be any picts within
  ;;   `base'. The top-left locations of `p-from' and `p-to'
  ;;   determine the placement of the top-left of `p'.
  (define (slide-pict base p p-from p-to n)
    (let-values ([(x1 y1) (fail-gracefully (lambda () (lt-find base p-from)))]
                 [(x2 y2) (fail-gracefully (lambda () (lt-find base p-to)))])
      (pin-over base
                (+ x1 (* (- x2 x1) n))
                (+ y1 (* (- y2 y1) n))
                p)))
  
  ;; Concatenate a sequence of animations
  (define (sequence-animations . l)
    (let ([len (length l)])
      (lambda (n)
        (cond
          [(zero? n)
           ((car l) 0.0)]
          [(= n 1.0)
           ((list-ref l (sub1 len)) n)]
          [else
           (let ([pos (inexact->exact (floor (* n len)))])
             ((list-ref l pos) (* len (- n (* pos (/ len))))))]))))
  
  ;; Reverse a sequence of animations
  (define (reverse-animations . l)
    (let ([s (apply sequence-animations l)])
      (lambda (n)
        (s (- 1 n)))))

  ;; Transition from one slide-centered content to another:
  (define (sweep from to)
    (lambda (n)
      (let ([span (blank client-w 0)])
        (if (n . < . 0.5)
            (let ([fg1 (ghost from)]
                  [fg2 (ghost from)])
              (slide-pict (cc-superimpose
                           (refocus (hc-append (+ 10 margin) fg2 span)
                                    span)
                           fg1)
                          from fg1 fg2 (zoom (* 2 n))))
            (let ([tg1 (ghost to)]
                  [tg2 (ghost to)])              
              (slide-pict (cc-superimpose
                           (refocus (hc-append (+ 10 margin) span tg1)
                                    span)
                           tg2)
                          to tg1 tg2 (zoom (* 2 (- n 0.5)))))))))

  ;; helper:
  (define (fail-gracefully t)
    (with-handlers ([exn:fail? (lambda (x) (values 0 0))])
      (t)))
 
  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;  Misc configuration
  
  (define LINE-WIDTH 2)
  (define link-arrow-size 16)
  (define SMALL-LINK-LEN 30)

  (define background (inset
                      (colorize
                       (filled-rectangle (+ client-w (* 2 margin))
                                         (+ client-h (* 2 margin)))
                       (make-object color% 240 240 190))
                      (- margin)))
  
  (current-title-color "darkblue")
  
  (let ([orig (current-slide-assembler)])
    (current-slide-assembler
     (lambda (t sep p)
       (let ([s (orig t sep p)])
         (ct-superimpose background
                         s)))))
  
  (define my-balloon-color "white")

  (define (npage-item . rest)
    (apply item (* 3/4 client-w) rest))
  (define (lpage-item . rest)
    (apply item (* 4/5 client-w) rest))
  
  (define (npage-para . rest)
    (apply para (* 3/4 client-w) rest))
  
  (define dw-tag-color "purple")
  
  (define const-col "forestgreen")
  
  (define-syntax define-definer 
    (syntax-rules ()
      [(_ define-consts const pfx)
       (...
        (define-syntax (define-consts stx)
          (syntax-case stx ()
            [(_ id)
             (with-syntax ([p:id (datum->syntax-object
                                  #'id
                                  (string->symbol (format pfx (syntax-e #'id)))
                                  #'id)])
               #'(define p:id (const 'id)))]
            [(_ id ...)
             #'(begin (define-consts id) ...)])))]))
  
  (define (const s)
    (colorize (text (symbol->string s) '(bold . modern) (current-font-size)) const-col))
  (define-definer define-consts const "p:~a")
  (define-consts call/cc call/comp abort prompt
    call/cm current-marks
    dw dynamic-wind
    set! begin
    list first
    handle catch throw
    handle-mark-key catch-tag fold default-exn-handler
    shift exit)

  (define server-const-col "forestgreen")
  
  (define (server-const s)
    (colorize (text (symbol->string s) '(bold . modern) (current-font-size)) server-const-col))
  (define-definer define-server-consts const "s:~a")
  (define-server-consts
    serve reply dispatch do-form cont->url
    generate-html)
  
  (define hole (colorize (parameterize ([code-colorize-enabled #f])
                           (code []))
                         "darkgray"))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; Drawing continuations

  (define (mk-down-link h)
    (linewidth LINE-WIDTH (inset (pip-arrow-line 0 h link-arrow-size) 0 0 0 h)))
  (define short-down-link (mk-down-link link-arrow-size))
  (define down-link (mk-down-link SMALL-LINK-LEN))
  (define long-down-link (mk-down-link (* 2 SMALL-LINK-LEN)))
  
  (define bubble-color "white")

  (define (square-ellipse w h squareness)
    (let ([p (new dc-path%)]
          [c-ness (* 0.5 (- 1.0 squareness))]
          [z 0.4])
      (send p move-to 0 (* h c-ness))
      (send p curve-to 0 (* h c-ness z) (* w c-ness z) 0 (* w c-ness) 0)
      (send p line-to (* w (- 1.0 c-ness)) 0)
      (send p curve-to (* w (- 1.0 (* c-ness z))) 0 w (* h c-ness z) w (* h c-ness))
      (send p line-to w (* h (- 1.0 c-ness)))
      (send p curve-to w (* h (- 1.0 (* c-ness z))) (* w (- 1.0 (* c-ness z))) h (* w (- 1.0 c-ness)) h)
      (send p line-to (* w c-ness) h)
      (send p curve-to (* w c-ness z) h 0 (* h (- 1.0 (* c-ness z))) 0 (* h (- 1.0 c-ness)))
      (send p close)
      (dc (lambda (dc x y)
            (send dc draw-path p x y))
          w h)))
  
  (define (interpolate n s e)
    (+ s (* (- e s) n)))
  
  (define (bubble** c n)
    (let ([w (interpolate n 
                          (* 1.1 (pict-width c))
                          (+ (pict-width c) 8))]
          [h (interpolate n 
                          (* 2.5 (pict-height c))
                          (+ (pict-height c) 28))])
      (baselines (cc-superimpose (linewidth LINE-WIDTH (square-ellipse w h n))
                                 c)
                 c)))
  
  (define (baselines p c)
    (let-values ([(x y) (lt-find p c)])
      (inset (refocus p c)
             x y 
             (- (pict-width p) x (pict-width c))
             (- (pict-height p) y (pict-height c)))))

  (define (bubble c)
    (bubble** c 0))
  
  (define (redex c)
    (bubble** c 1))
    
  (define (spur c)
    (let ([w (* 1.8 (pict-width c))]
          [h (* 1.8 (pict-height c))])
      (cc-superimpose (colorize (filled-flash w h 16 0.3) "yellow")
                      (linewidth LINE-WIDTH (outline-flash w h 16 0.3))
                      (inset c 0 0 0 3))))

  (define chain*
    (case-lambda
      [(link redex) redex]
      [(link bubble next . rest)
       (if (number? next)
           (vc-append 0
                      bubble
                      (cellophane
                       (vc-append 0
                                  link
                                  (apply chain* link rest))
                       next))
           (apply chain* link bubble 1.0 next rest))]
      [(link bubble . rest)
       (vc-append 0
                  bubble
                  link
                  (apply chain* link rest))]))
  
  (define (chain . rest)
    (apply chain* down-link rest))
  
  (define (chain/arrow-val n . rest)
    (apply chain* (cellophane down-link n) rest))

  (define (tall-chain . rest)
    (apply chain* long-down-link rest))
  
  (define (mv v)
    (lambda (sub)
      (let ([v (text v `(italic . ,(current-code-font)) (current-font-size))])
        (if sub
            (hbl-append v
                        (text (format "~a" sub) `(subscript italic . ,(current-code-font)) (current-font-size)))
            v))))
  
  (define v (mv "v"))
  (define -e (mv "e"))
  (define (e n)
    (-e (case n
          [(3) 5]
          [(4) 6]
          [(5) 3]
          [(6) 4]
          [else n])))
  (define x (mv "x"))
  
  (define-values (bad-call/cc call/cc-puzzle)
    (values
     (code (λ (f) (#,p:shift k  (k (f (λ (x) (#,p:shift c  (k x))))))))
     (code ((λ (c) ((#,p:call/cc  #,p:call/cc) c))  (#,p:call/cc  #,p:call/cc)))))
  
  (define chain1-node1-code (code (#,(v 1) #,hole)))
  (define chain1-node1 (bubble chain1-node1-code))
  
  (define chain1-node2-code (code (#,hole #,(v 2))))
  (define chain1-node2 (bubble chain1-node2-code))
  
  (define chain1-before-body
    (code ((λ (x) x) #,(v 3))))

  (define chain1
    (chain chain1-node1
           chain1-node2
           (redex chain1-before-body)))

  (define (zoom n)
    (cond
      [(= n 0.0) 0.0]
      [(= n 1.0) 1.0]
      [else (- 0.5 (/ (cos (* pi n)) 2))]))
  

  (define (mk-arrow-pict sz curvy?)
    (let ([f (make-arrow-pict sz curvy? 'roman (current-font-size))])
      (lambda ()
        (linewidth LINE-WIDTH (f)))))

  (define long-arrow-pict (mk-arrow-pict "xxx" 'straight))
  (define curvy-arrow-pict (mk-arrow-pict "xxx" 'curvy))
  
  (define (eval-step a b)    
    (hc-append 20
               a
               (long-arrow-pict)
               b))
  
  (define (tight-eval-step a b)
    (eval-step (inset a 0 0 (- 20) 0)
               (inset b (- 20) 0 0 0)))
  
  (define (r-tighten p) (inset p 0 0 (- 15) 0))
  (define (l-tighten p) (inset p (- 25) 0 0 0))

  (define (cont e) e)
  (define (comp/link e down-link) 
    (let ([p (inset e 0 (pict-height down-link) 0 0)])
      (pin-over p
                p ct-find
                (colorize down-link "forestgreen"))))
  (define (comp e)
    (comp/link e short-down-link))
  (define ((comp-mid e) n)
    (comp/link e (if (zero? n)
                     (blank)
                     (scale short-down-link n))))
  
  (define (leads-to a b)
    (refocus (hbl-append 2
                         a
                         (curvy-arrow-pict)
                         b)
             a))
  
  
  (define (leads-to*/lt leads-to n a b)
    (lt-superimpose
     a
     (cellophane (leads-to (ghost a) b)
                 n)))
  
  (define (leads-to* n a b)
    (leads-to*/lt leads-to n a b))
  
  (define F-op (text "F" 'script (current-font-size)))

  ;; ----------------------------------------
  
  (define chain1-after-redex
    (redex (code (#,(v 3) #,(v 2)))))

  (define chain1-after
    (chain chain1-node1
           chain1-after-redex))
  
  (define chain1-eval
    (eval-step chain1
               chain1-after))
  
  (define (chain1-mid-x/body body lt-n leads-to n)
    (let* ([v3 (v 3)]
           [g1 (ghost v3)]
           [g2 (ghost v3)]
           [r (redex body)])
      (slide-pict (chain chain1-node1
                         (bubble** (code (#,(fade-pict n hole g2) #,(v 2))) n)
                         (- 1.0 n)
                         (leads-to*/lt leads-to lt-n r g1))
                  (cellophane v3 lt-n) g1 g2 (zoom n))))
  
  (define ((chain1-mid-x lt-n) n)
    (chain1-mid-x/body chain1-before-body lt-n leads-to n))

  (define chain1-mid (chain1-mid-x 1.0))

  
  (define orig-redex
    (code ((λ (x) x) #,(v 3))))
  
  (define orig-expr
    (code (#,(v 1) (#,orig-redex #,(v 2)))))
  
  (define (evenize p)
    ;; Make sure p has an even width and height, so that a centering
    ;;  combination with another picture doesn't change the alignment
    ;;  of the other picture.
    (let ([fix (lambda (v)
                 (- (if (even? (ceiling v))
                        (ceiling v)
                        (+ 1 (ceiling v)))
                    v))])
      (inset p 
             (fix (pict-width p))
             (fix (pict-height p))
             0 0)))
  
  (define (fade-level n)
    (+ 0.3 (* (- 1.0 n) 0.7)))
  
  (define ((orig-defn-steps-mid mode) n)
    (let* ([inner
            (case mode 
              [(hilite-inner copy-inner) orig-redex]
              [(hilite-middle)
               (cellophane orig-redex (fade-level n))]
              [(finish)
               (cellophane orig-redex (fade-level (- 1.0 n)))]
              [else
               (cellophane orig-redex (fade-level 1.0))])]
           [ig (ghost inner)])
      (let* ([pre-mid (code (#,ig #,(v 2)))]
             [mid (case mode
                    [(hilite-inner hilite-outer)
                     (cellophane pre-mid (fade-level n))]
                    [(copy-inner copy-outer)
                     (cellophane pre-mid (fade-level 1.0))]
                    [(hilite-middle finish)
                     (cellophane pre-mid (fade-level (- 1.0 n)))]
                    [(copy-middle)
                     pre-mid])]
             [mg (ghost mid)])
        (let* ([pre-out (code (#,(v 1) #,mg))]
               [out (case mode
                      [(hilite-inner)
                       (cellophane pre-out (fade-level n))]
                      [(copy-inner hilite-middle copy-middle)
                       (cellophane pre-out (fade-level 1.0))]
                      [(hilite-outer)
                       (cellophane pre-out (fade-level (- 1.0 n)))]
                      [(copy-outer finish)
                       pre-out])])
          (let ([base (pin-over
                       (pin-over out
                                 mg lt-find
                                 mid)
                       ig lt-find
                       inner)]
                [1g (ghost chain1-node1)]
                [2g (ghost chain1-node2)]
                [3g (ghost (redex orig-redex))])
            (let ([skel
                   (hc-append 20
                              base
                              (if (eq? mode 'finish)
                                  (cellophane (code =) n)
                                  (ghost (code =)))
                              (chain/arrow-val (if (eq? mode 'finish)
                                                   n
                                                   0.0)
                                               (case mode
                                                 [(finish) chain1-node1]
                                                 [else 1g])
                                               (case mode
                                                 [(init hilite-inner copy-inner
                                                        hilite-middle copy-middle)
                                                  2g]
                                                 [else chain2-node2])
                                               (case mode
                                                 [(init hilite-inner copy-inner)                                           
                                                  3g]
                                                 [else (redex orig-redex)])))])
              (case mode
                [(copy-inner) (slide-pict skel
                                          (fade-pict n inner (redex orig-redex))
                                          inner 3g n)]
                [(copy-middle) (slide-pict skel
                                           (let* ([a (fade-pict n ig hole)]
                                                  [p (code (#,a #,(v 2)))])
                                             (cc-superimpose
                                              (fade-pict n (blank) (bubble (ghost p)))
                                              p)) 
                                           mid 2g n)]
                [(copy-outer) (slide-pict skel
                                          (let* ([a (fade-pict n mg hole)]
                                                 [p (code (#,(v 1) #,a))])
                                             (cc-superimpose
                                              (fade-pict n (blank) (bubble (ghost p)))
                                              p))
                                          out 1g n)]
                [else skel])))))))
                
  
  (define ((chain1-defn-mid-x lt-n) n)
    (let* ([chain1-x ((chain1-mid-x lt-n) 0.0)]
           [g1 (ghost chain1-x)]
           [g2 (ghost chain1-x)])
      (slide-pict (cellophane
                   (cc-superimpose
                    (evenize
                     (hc-append 20
                                orig-expr
                                (code =)
                                g1))
                    g2)
                   (- 1.0 n))
                  chain1-x g1 g2 (zoom n))))
  
  (define chain1-defn-mid (chain1-defn-mid-x 0.0))
  (define chain1-defn-leads-mid (lambda (n)
                                  ((chain1-defn-mid-x n) 1.0)))
  
  (define chain1-un-mid
    (reverse-animations chain1-defn-leads-mid
                        chain1-mid))

  (define chain1-defn (chain1-defn-mid 0))
  
  (define chain2-node1-code chain1-node1-code)
  (define chain2-node1 chain1-node1)
  (define chain2-node2-code chain1-node2-code)
  (define chain2-node2 chain1-node2)

  (define (realign together top)
    (lift together
          (- (let-values ([(x y) (find-lbl together top)])
               (- y (pict-descent together))))))
  
  (define chain2-k
    (let ([top chain2-node1-code])
      (let ([together (chain chain2-node1
                             chain2-node2)])
        (realign together top))))

  (define (chain2-second form)
    (chain chain2-k
           (redex (code (#,(form chain2-k) #,(v 3))))))

  (define (chain2-before-body-x inner)
    (code (#,p:call/cc (λ (k) #,inner))))
  
  (define chain2-before-body
    (chain2-before-body-x (code (k #,(v 3)))))

  (define chain2
    (chain chain2-k (redex chain2-before-body)))
  
  (define chain2-eval
    (eval-step
     chain2
     (chain2-second cont)))

  (define (c-evenize p)
    (cc-superimpose (evenize (launder (ghost p)))
                    p))
  
  (define (chain1-to-chain2-mid n)
    (chain1-mid-x/body 
     (c-evenize (fade-pict n chain1-before-body chain2-before-body))
     0.0
     leads-to
     0.0))
  
  (define ((chain2-capture-mid-x chain2 chain2-k orig-chain2-k) n)
    (let-values ([(x y) (lt-find chain2 orig-chain2-k)])
      (pin-over chain2
                (+ x (* (zoom n) (pict-width chain2-k) 1.5))
                (+ y (* (zoom n) (pict-height chain1-node1) -0.5))
                (cellophane chain2-k n))))
  
  (define chain2-capture-mid
    (chain2-capture-mid-x chain2 chain2-k  chain2-k))
  
  (define (chain2-shift-mid n)
    (let* ([cg2 (ghost chain2-k)]
           [inner-app (code (#,(fade-pict* ctl-superimpose n (code k) cg2) #,(v 3)))])
      (let ([g1 (ghost inner-app)]
            [g2 (ghost inner-app)]
            [cg1 (ghost chain2-k)]
            [orig-chain2-k (inset chain2-k 0)])
        (slide-pict
         ((chain2-capture-mid-x
           (slide-pict
            (chain orig-chain2-k
                   (redex (fade-pict* ctl-superimpose
                                      n
                                      (chain2-before-body-x g1)
                                      g2)))
            inner-app
            g1 g2 n)
           cg1 orig-chain2-k) 1.0)
         chain2-k cg1 cg2 n))))
  
  (define (chain2-leave-mid n)
    (chain (cellophane chain2-k (- 1 n))
           (redex (code (#,chain2-k #,(v 3))))))

  (define (chain2-restore-mid n)
    (let ([g1 (ghost chain2-k)]
          [g2 (ghost chain2-k)])
      (slide-pict
       (chain g2
              (redex (code (#,g1 #,(v 3)))))
       chain2-k g1 g2 n)))
  
  (define (chain2-reduce-mid n)
    (let ([g (ghost (v 3))])
      (chain1-mid-x/body (code (#,(ghost chain2-k) #,g))
                         1.0
                         (lambda (x y)
                           (pin-over x
                                     g lt-find
                                     y))
                         n)))
  
  (define chain2-third
     (chain (bubble (code (#,(v 1) #,hole)))
            (redex (code (#,(v 3) #,(v 2))))))
  
  (define chain2-eval-eval
    (eval-step
     (chain2-second cont)
     chain2-third))

  (define chain2-eval-eval-comp
    (eval-step
     (chain2-second comp)
     (chain chain2-k
            chain2-third)))
  
  (define (chain2-to-comp-mid n)
    (chain chain2-k
           (redex (code (#,((comp-mid chain2-k) n) #,(v 3))))))
  
  (define ((splice-cont-mid pre-k appl-k) n)
    (let ([o1 (ghost appl-k)]
          [g1 (ghost appl-k)])
      (slide-pict (chain pre-k
                         (vc-append
                          0
                          (fade-pict n (blank) (vc-append g1 down-link))
                          (redex (code (#,(fade-pict n (comp o1) (ghost hole)) #,(v 3))))))
                  appl-k o1 g1 n)))
  
  (define chain2-comp-app-mid
    (splice-cont-mid chain2-k chain2-k))
  
  (define exit-node (bubble (code (#,p:exit #,hole))))
  
  (define (chain2-comp-finish-mid-x n end-n middle-n)
    (let* ([v3 (v 3)]
           [g1 (ghost v3)]
           [g2 (ghost v3)])
      (slide-pict (vc-append 
                   (fade-pict end-n 
                              (blank) 
                              (chain exit-node
                                     (blank)))
                   (chain (vc-append 
                           chain2-k
                           (fade-pict middle-n
                                      (blank)
                                      (vc-append down-link
                                                 exit-node)))
                          chain1-node1
                          (bubble** (code (#,(fade-pict n hole g2) #,(v 2))) n))
                   (fade-pict end-n
                              (cellophane (vc-append
                                           down-link
                                           (redex (code (#,(ghost hole) #,g1))))
                                          (- 1 n))
                              (blank)))
                  v3 g1 g2 n)))
  
  (define (chain2-comp-finish-mid n)
    (chain2-comp-finish-mid-x n 0.0 0.0))
  
  (define (chain2-comp-add-mid n)
    (chain2-comp-finish-mid-x 1.0 n 0.0))

  (define (chain2-comp-addadd-mid n)
    (chain2-comp-finish-mid-x 1.0 1.0 n))

  (define chain3-k
    (chain (bubble (code (#,p:prompt #,hole)))
           chain2-node2))
  
  (define (chain3-mid copy-n drop-n app-n abort-n abort-drop-n jump-n)
    (let* ([cont (comp chain2-node2)]
           [g2 (launder (ghost cont))]
           [g3 (launder (ghost cont))]
           [g4 (launder (ghost cont))]
           [vg1 (ghost v3)]
           [vg2 (ghost v3)])
      (slide-pict
       (slide-pict
        (slide-pict (let ([p (chain #; exit-node
                                    (bubble** (code (#,(v 1) #,(fade-pict jump-n hole vg2)))
                                              jump-n)
                                    (- 1 abort-drop-n)
                                    chain3-k
                                    (let* ([k (fade-pict app-n (code k) g3)]
                                           [body (code (#,k #,(v 3)))]
                                           [body-g (launder (ghost body))]
                                           [body-g2 (launder (ghost body))])
                                      (slide-pict
                                       (redex (evenize
                                               (fade-pict
                                                abort-n
                                                (fade-pict
                                                 app-n
                                                 (cellophane
                                                  (code (#,p:call/comp (λ (k) #,body-g)))
                                                  (- 1 drop-n))
                                                 body-g2)
                                                (code (#,p:abort #,vg1)))))
                                       (cellophane body (- 1 abort-n))
                                       body-g body-g2 app-n)))])
                      (refocus (hc-append (cc-superimpose p
                                                          (blank (* client-w 0.6) 0))
                                          g2)
                               p))
                    g4 chain2-node2 g2 copy-n)
        (cellophane (cellophane cont copy-n) (- 1 abort-n)) g4 g3 app-n)
       (cellophane v3 abort-n) vg1 vg2 jump-n)))
  
  (define (chain3-copy-mid n)
    (chain3-mid n 0.0 0.0 0.0 0.0 0.0))
  (define (chain3-app-mid n)
    (chain3-mid 1.0 n n 0.0 0.0 0.0))

  (define (chain3-to-abort-mid n)
    (chain3-mid 1.0 1.0 1.0 n 0.0 0.0))
  (define (chain3-drop-mid n)
    (chain3-mid 1.0 1.0 1.0 1.0 n 0.0))
  (define (chain3-jump-mid n)
    (chain3-mid 1.0 1.0 1.0 1.0 1.0 n))
  
  ;; ----------------------------------------
  
  (define (hilite-dot n p)
    (refocus (cc-superimpose
              (cellophane
               (colorize
                (filled-ellipse (* 1.3 (pict-width p))
                                (* 1.3 (pict-height p)))
                (make-object color% 255 200 200))
               n)
              p)
             p))

  ;; => An OK Example
  ;;  (but currently cut from the talk)
  (define (chain5-mid comp-hilite-n copy-n app-n abort-hilite-n drop-n abort-n)
    (let* ([cont (comp (realign chain2-node2 chain2-node2-code))]
           
           [g2 (launder (ghost cont))]
           [g3 (launder (ghost cont))]
           [g4 (launder (ghost cont))]
           [g8 (launder (ghost cont))]
           [g9 (launder (ghost cont))]
           
           [hilite (lambda (n p)
                     (hilite-dot n p))]
           
           [v4 (hilite abort-hilite-n (v 4))]
           
           [inner (code (#,p:abort #,v4 #,(fade-pict app-n
                                                     (code k)
                                                     g3)))]
           [inner-g1 (launder (ghost inner))]
           [inner-g2 (launder (ghost inner))]
           
           [v5 (hilite comp-hilite-n (v 5))]
                      
           
           [body (slide-pict
                  (fade-pict
                   app-n
                   (code (#,p:call/comp (λ (k) #,inner-g1)
                                        #,v5))
                   inner-g2)
                  inner inner-g1 inner-g2 app-n)]
           
           [shrink (lambda (p)
                     (if (zero? abort-n)
                         p
                         (scale p (- 1 (* 0.6 abort-n)))))]

           [handler (v 10)]
           [hg1 (ghost handler)]
           [hg2 (ghost handler)]
           
           [p (chain (bubble** (fade-pict
                                abort-n
                                (cellophane (code (#,p:prompt #,v4 #,hole #,hg1))
                                            (- 1 drop-n))
                                (code (#,hg2 #,g8)))
                               abort-n)
                     (- 1 drop-n)
                     (shrink chain2-node1)
                     (shrink (bubble (code (#,p:prompt #,v5 #,hole #,(v 11)))))
                     chain2-node2
                     (shrink (redex body)))]
           [p (refocus (hc-append 10 p g2) p)]
           [p (slide-pict p
                          g4 chain2-node2 g2 copy-n)]
           [p (slide-pict p
                          g9 g4 g3 app-n)]
           [p (slide-pict p
                          (cellophane cont copy-n) g9 g8 abort-n)]
           [p (slide-pict p
                          handler hg1 hg2 abort-n)])
      (inset p 0 20 0 0)))

  (define (chain5-comp-hilite-mid n)
    (chain5-mid n 0.0 0.0 0.0 0.0 0.0))

  (define (chain5-copy-mid n)
    (chain5-mid 1.0 n 0.0 0.0 0.0 0.0))

  (define (chain5-app-mid n)
    (chain5-mid (- 1 n) 1.0 n 0.0 0.0 0.0))

  (define (chain5-abort-hilite-mid n)
    (chain5-mid 0.0 1.0 1.0 n 0.0 0.0))

  (define (chain5-drop-mid n)
    (chain5-mid 0.0 1.0 1.0 1.0 n 0.0))

  (define (chain5-abort-mid n)
    (chain5-mid 0.0 1.0 1.0 1.0 1.0 n))

  ;; ----------------------------------------
  
  ;; => An OK example
  ;;  (but currently cut from the talk)
  (define (chain6-mid q:prompt q:abort drop-n app-n F-n insert-n tagged-n hilite-n)
    (let* ([proc (code (λ (x) #,(e 1)))]
           
           [pg1 (launder (ghost proc))]
           [pg2 (launder (ghost proc))]
           
           [vg1 (ghost v3)]
           [vg2 (ghost v3)]
           
           [tagged (lambda (p t)
                     (if (zero? tagged-n)
                         p
                         (hbl-append p
                                     (fade-pict tagged-n
                                                (tt "")
                                                (hb-append (tt " ") 
                                                           (if (equal? t "exn")
                                                               (hilite-dot hilite-n (v t))
                                                               (v t)))))))]
      
           [p (chain chain2-node1                     
                     (bubble** (fade-pict
                                app-n
                                (cellophane
                                 (code (#,(tagged q:prompt "exn") #,hole #,pg1))
                                 (- 1 drop-n))
                                (code (#,pg2 #,vg2)))
                               app-n)
                     (- 1 drop-n)
                     (vc-append chain2-node2
                                (fade-pict insert-n
                                           (blank)
                                           (vc-append down-link
                                                      (bubble (code (#,(tagged p:prompt 0) #,hole #,(v 12)))))))
                     (redex (code (#,(tagged q:abort "exn") #,vg1))))]

           [p (slide-pict p
                          proc pg1 pg2 app-n)]
           [p (slide-pict p
                          v3 vg1 vg2 app-n)]
           
           [p (cb-superimpose
               p
               (cellophane
                (vc-append
                 gap-size
                 (htl-append (code (#,p:prompt #,(e 1)))
                             (tt " = ")
                             (code (#,p:prompt #,(e 1) (λ (t) (t)))))
                 (htl-append F-op
                             (tt " = ")
                             (code (λ (f) (#,p:call/comp
                                           (λ (k) (#,p:abort (λ () (f k)))))))))
                F-n))])
               
      p))
  
  (define (chain6-drop-mid n)
    (chain6-mid p:prompt p:abort n 0.0 0.0 0.0 0.0 0.0))
  
  (define (chain6-app-mid n)
    (chain6-mid p:prompt p:abort 1.0 n 0.0 0.0 0.0 0.0))

  (define (chain6-F-mid n)
    (chain6-mid p:prompt p:abort 1.0 1.0 n 0.0 0.0 0.0))
  
  (define (chain6-to-catch-mid n)
    (chain6-mid (fade-pict n p:prompt p:catch)
                (fade-pict n p:abort p:throw)
                0.0 0.0 0.0 0.0 0.0 0.0))

  (define (chain6-insert-mid n)
    (chain6-mid p:catch p:throw 0.0 0.0 0.0 n 0.0 0.0))

  (define (chain6-amb-mid n)
    (chain6-mid (fade-pict n p:catch p:prompt)
                (fade-pict n p:throw p:abort)
                0.0 0.0 0.0 1.0 0.0 0.0))

  (define (chain6-tagged-mid n)
    (chain6-mid p:prompt p:abort 0.0 0.0 0.0 1.0 n 0.0))

  (define (chain6-hilite-mid n)
    (chain6-mid p:prompt p:abort 0.0 0.0 0.0 1.0 1.0 n))

  ;; ----------------------------------------
  ;; WCM
  
  (define (mark node binding)
    (refocus (pin-over (ht-append (ghost (inset binding 0 (- 20 (pict-height binding)) 10 0))
                                  (inset node (/ (pict-width node) -2) 0))
                       binding lt-find
                       binding)
             node))
  
  (define bind-arrow (colorize (arrowhead link-arrow-size 0.0) "blue"))
  
  (define (bind* . binds)
    (let ([binds (let loop ([binds binds])
                   (if (null? binds)
                       null
                       (list* (car binds)
                              bind-arrow
                              (cadr binds)
                              (loop (cddr binds)))))])
      (let ([t (table 3
                      binds
                      (list* rbl-superimpose cbl-superimpose lbl-superimpose)
                      cbl-superimpose
                      3 3)])
        (let ([b (inset t 2)])
          (let ([w (+ (pict-width b) 5)]
                [h (pict-height b)])
            (values
             (cc-superimpose
              (colorize (filled-rounded-rectangle w h)
                        "lightblue")
              (linewidth LINE-WIDTH (rounded-rectangle w h))
              b)
             t))))))
  
  (define (bind . binds)
    (let-values ([(b t) (apply bind* binds)])
      b))
  
  (define example-binding (bind (v 7) (v 8)))
  
  (define chain7-k
    (chain chain1-node1
           (mark chain2-node2
                 (bind (v 7) (v 8)))))

  (define chain7-eval
    (eval-step
     (chain chain7-k
            (bubble (code (#,p:first #,hole)))
            (mark (redex (code (#,p:current-marks #,(v 7))))
                  (bind (v 7) (v 9))))
     (chain chain7-k
            (redex (code (#,p:first (#,p:list #,(v 9) #,(v 8))))))))
  
  ;; ----------------------------------------
  
  ;; => An OK example
  ;;  (but currently cut from the talk)
  (define (chain8-mid mark1-n step1-n mark2-n comp-n copy-n)
    (let* ([body3 (cellophane (e 1) (- 1 comp-n))]
           [b3-g1 (launder (ghost body3))]
           [b3-g2 (launder (ghost body3))]
           
           [v7 (v 7)]
           [v7-g1 (ghost v7)]
           [v7-g2 (ghost v7)]
           [v12 (v 12)]
           [v12-g1 (ghost v12)]
           [v12-g2 (ghost v12)]
           
           [body2
            (code (#,p:call/cm #,v7-g1 #,v12-g1 (λ () #,b3-g1)))]
           [b2-g1 (launder (ghost body2))]
           [b2-g2 (launder (ghost body2))]
           
           [body1
            (code (#,p:prompt #,(fade-pict step1-n b2-g1 hole)))]
           [b1-g1 (launder (ghost body1))]
           [b1-g2 (launder (ghost body1))]
           
           [v10 (v 10)]
           [v10-g1 (ghost v10)]
           [v10-g2 (ghost v10)]
           [v11 (v 11)]
           [v11-g1 (ghost v11)]
           [v11-g2 (ghost v11)]
           
           [orig (code (#,p:call/cm
                        #,v10-g1 #,v11-g1
                        (λ () #,b1-g1)))]
           
           [capturable
            (bubble** (fade-pict comp-n
                                 (fade-pict mark2-n
                                            b2-g2
                                            b3-g2)
                                 (code (#,hole #,v3)))
                      (- 1 comp-n))]
           [mk-captured-node
            (lambda (v7 v12)
              (mark capturable
                    (cellophane
                     (bind v7 v12)
                     mark2-n)))]
           [captured-node (mk-captured-node v7-g2 v12-g2)]
           [copied-node (mk-captured-node v7 v12)]
           [cn-g1 (launder (ghost captured-node))]

           [p (chain chain1-node1   
                     (mark (bubble** (fade-pict mark1-n
                                                orig
                                                b1-g2)
                                     (- 1 step1-n))
                           (cellophane
                            (bind v10-g2 v11-g2)
                            mark1-n))
                     step1-n
                     captured-node
                     comp-n
                     (redex
                      (code (#,p:call/comp (λ (k) (#,(v 13) k))))))]

           [p (refocus (hc-append p cn-g1) p)]
           
           [p (slide-pict p v10 v10-g1 v10-g2 mark1-n)]
           [p (slide-pict p v11 v11-g1 v11-g2 mark1-n)]
           
           [p (slide-pict p body1 b1-g1 b1-g2 mark1-n)]
           
           [p (slide-pict p (cellophane body2 (- 1 mark2-n))
                          b2-g1 b2-g2 step1-n)]
           
           [p (slide-pict p v7 v7-g1 v7-g2 mark2-n)]
           [p (slide-pict p v12 v12-g1 v12-g2 mark2-n)]

           [p (slide-pict p body3 b3-g1 b3-g2 mark2-n)]
           
           [p (slide-pict p (cellophane copied-node copy-n)
                          captured-node cn-g1 copy-n)])
      p))

  (define (chain8-mark1-mid n)
    (chain8-mid n 0.0 0.0 0.0 0.0))
  (define (chain8-step1-mid n)
    (chain8-mid 1.0 n 0.0 0.0 0.0))
  (define (chain8-mark2-mid n)
    (chain8-mid 1.0 1.0 n 0.0 0.0))
  (define (chain8-comp-mid n)
    (chain8-mid 1.0 1.0 1.0 n 0.0))
  (define (chain8-copy-mid n)
    (chain8-mid 1.0 1.0 1.0 1.0 n))
      
  ;; ----------------------------------------

  (define-values (chain9-bind1 chain9-bind1-table) (bind* (v 7) (v 9)))
  (define chain9-node3 (bubble (code (#,hole #,(v 3)))))

  (define chain9-captured-k
    (chain (mark chain9-node3
                 chain9-bind1)))
  
  (define chain9-k
    (chain chain1-node1
           (mark (bubble (code (#,p:prompt #,hole)))
                 (bind (v 7) (v 8)))
           chain9-captured-k))
  
  (define chain9-redex-bind
    (bind (v 7) (v 10)))
    
  (define chain9-eval
    (tight-eval-step
     (chain chain9-k
            (mark (redex (code (#,p:call/comp (λ (x) (#,(v 13) x)))))
                  chain9-redex-bind))
     (chain chain9-k
            (mark (redex (code (#,(v 13) #,(panorama (comp chain9-captured-k)))))
                  chain9-redex-bind))))
  
  (define chain10-eval
    (eval-step
     (r-tighten
      (tall-chain chain1-node1
                  (mark (redex (code (#,(panorama (comp (chain chain9-captured-k chain1-node2))) #,(v 14))))
                        (bind (v 7) (v 8)
                              (v 15) (v 10)))))
     (inset (chain (tall-chain chain1-node1
                               (mark (bubble (code (#,hole #,(v 3))))
                                     (bind (v 7) (v 9)
                                           (v 15) (v 10))))
                   (redex (code (#,(v 14) #,(v 2)))))
            0 0 0 20)))

  (define chain11-eval
    (tight-eval-step
     (chain chain9-k
            (mark (redex (code (#,p:current-marks #,(v 7))))
                  chain9-redex-bind))
     (chain chain1-node1
            (mark (bubble (code (#,p:prompt #,hole)))
                  (bind (v 7) (v 8)))
            (mark (redex (code ((#,p:list #,(v 10) #,(v 9)) #,(v 3))))
                  chain9-bind1))))
  
  ;; ----------------------------------------
  ;; dynamic-wind
  
  (define-struct dw (all in body out))
  
  (define (dw-node* n in body out)
    (let ([in (spur in)]
          [out (spur out)]
          [body-g (ghost (launder body))])
      (pin-over (cellophane
                  (pin-line
                   (pin-line (ht-append 5
                                        (inset in (max 0 (- (pict-width out) (pict-width in))) 0 0 0)
                                        (hb-append 
                                         5
                                         body-g
                                         (inset out 0 0 (max 0 (- (pict-width in) (pict-width out))) 0)))
                             in rc-find
                             body-g lc-find
                             LINE-WIDTH)
                   body-g rc-find
                   out lc-find
                   LINE-WIDTH)
                  n)
                 body-g lt-find
                 body)))

  (define (dw-node** n bubble-n in body out)
    (make-dw (dw-node* n in (bubble** body bubble-n) out)
             in
             body 
             out))
  
  (define (dw-node in body out)
    (dw-node** 1.0 0.0 in body out))
  
  (define dw-color "orange")
  
  (define curve-through 
    (case-lambda
      [(p pt1 . rest)
       (define path (new dc-path%))
       (define ct
         (case-lambda
           [(p) p]
           [(p pt) p]
           [(p pt1 pt2 . rest)
            (let-values ([(dir pt2 rest)
                          (if (memq pt2 '(cw ccw))
                              (values pt2 (car rest) (cdr rest))
                              (values 'ccw pt2 rest))])
              (let-values ([(x1 y1) (fail-gracefully (lambda () (cc-find p pt1)))]
                           [(x2 y2) (fail-gracefully (lambda () (cc-find p pt2)))])
                (let ([x2 (if (null? rest)
                              (+ x2 0.5)
                              x2)])
                  (if (eq? dir 'ccw)
                      (send path curve-to 
                            (/ (+ x1 x2) 2) y1
                            x2 (/ (+ y1 y2) 2)
                            x2 y2)
                      (send path curve-to 
                            x1 (/ (+ y1 y2) 2)
                            (/ (+ x1 x2) 2) y2
                            x2 y2)))
                (apply ct p pt2 rest)))]))
       (let-values ([(x1 y1) (fail-gracefully (lambda () (cc-find p pt1)))])
         (send path move-to x1 y1))
       (apply ct p pt1 rest)
       (cc-superimpose p
                       (linewidth
                        (* 2 LINE-WIDTH)
                        (colorize
                         (dc (lambda (dc x y)
                               (let ([b (send dc get-brush)])
                                 (send dc set-brush "black" 'transparent)
                                 (send dc draw-path path x y)
                                 (send dc set-brush b)))
                             (pict-width p)
                             (pict-height p))
                         dw-color)))]
      [(p) p]))

  ;; draws a patial curve-through, based on `n'
  (define (curve-through-mid n arrow g1 base . path)
    (if (zero? n)
        base
        (let* ([steps (sub1 (length (filter (lambda (x) (not (symbol? x))) path)))]
               [cnt (floor (* n steps))]
               [g2 (ghost arrow)]
               [g3 (ghost arrow)]
               [m-ct-find (lambda (p p2)
                            (let-values ([(x y) (fail-gracefully (lambda () (ct-find p p2)))])
                              (values (- x (/ (pict-width arrow) 2)) y)))])
          (let ([new-steps
                 (let loop ([steps path][cnt (add1 cnt)])
                   (cond
                     [(zero? cnt) null]
                     [(symbol? (car steps)) (cons (car steps) (loop (cdr steps) cnt))]
                     [else (cons (car steps) (loop (cdr steps) (sub1 cnt)))]))])
            (let-values ([(first-pict dir last-pict)
                          (let loop ([steps path][cnt cnt][arrow? #f])
                            (cond
                              [(null? steps) (values g1 #f g1)]
                              [(symbol? (car steps))
                               (if (and (zero? cnt) arrow?)
                                   (let-values ([(f s l) (loop (cdr steps) 0 #f)])
                                     (values f (car steps) l))
                                   (loop (cdr steps) cnt arrow?))]
                              [(zero? cnt) (let-values ([(f d l) (loop (cdr steps) 0 #t)])
                                             (values (car steps) d f))]
                              [else (loop (cdr steps) (sub1 cnt) arrow?)]))])
              (apply curve-through
                     (slide-pict (pin-over (pin-over base
                                                     last-pict m-ct-find
                                                     g2)
                                           first-pict m-ct-find
                                           g3)
                                 ((if (= 1 n) values ghost) arrow)
                                 g3 g2 (/ (- n (* cnt (/ steps)))
                                          (/ steps)))
                     (append new-steps
                             (if dir (list dir) null)
                             (list arrow))))))))
  
  (define etc.-arrow-size link-arrow-size)
  (define etc. (colorize (arrowhead etc.-arrow-size (/ pi 2)) dw-color))
  
  (define (dw-inset p) (inset p 5 0))
  
  (define v3 (v 3))
  (define D3-midpt (blank))
  (define D4-tag (blank))    
  (define hole-etc. (inset (colorize (arrowhead etc.-arrow-size (* pi -0.2)) dw-color)
                           0 0 2 0))
  (define ghost-hole-etc. (ghost hole-etc.))

  (define (outline-box b)
    (refocus (cc-superimpose (linewidth LINE-WIDTH (dash-frame (ghost b) 3))
                             b)
             b))

  ;; This bit of abstraction is a leftover from the paper figures.
  ;; Unfortunately, it cuts in the wrong dimension compared to
  ;; what's needed for animations, so it makes a mess later.
  (define (make-chainDs tagged? dfnt? comp?)

    (define (make-chainD1-dw2 tag suffix)
      (dw-node (code #,(e 1))
               (if tag
                   (code #,p:dw #,tag)
                   (code #,(dw-inset (hbl-append p:dw suffix))))
               (code #,(e 2))))
    
    (define chainD1-dw2 (make-chainD1-dw2 (and tagged? (x 1)) (blank)))
    (define chainD1-dw2-alt (if dfnt?
                                (make-chainD1-dw2 (and tagged? (x 4)) (blank))
                                chainD1-dw2))
    
    (define chainD1-node3
      (redex (code (#,p:abort #,(v 4) #,v3))))
    
    (define (make-chainD1 base)
      (curve-through (let ([p base])
                       (let-values ([(x y) (cc-find p (dw-out chainD1-dw2))])
                         (pin-over p (- x (/ (pict-width etc.) 2)) 0 etc.)))
                     chainD1-node3
                     (dw-out chainD1-dw2)
                     etc.))
    
    (define chainD1 (make-chainD1 (chain chain1-node1
                                         (dw-all chainD1-dw2)
                                         chainD1-node3)))
    (define chainD1+ (make-chainD1 (chain chain1-node1
                                          (dw-all chainD1-dw2)
                                          chain1-node2
                                          chainD1-node3)))
    
    (define chainD2-node1 (bubble (code (#,(v 6) #,hole))))
    (define chainD2-node3 chain1-node2)
    (define incoming (blank))
    
    (define (chainD2-k-mid n)
      (let ([g1 (ghost hole-etc.)])
        (realign (curve-through-mid (zoom n) hole-etc. g1
                                    (let ([p (chain chain1-node1
                                                    (dw-all chainD1-dw2)
                                                    (pin-over chainD2-node3
                                                              chainD2-node3 (lambda (p p2)
                                                                              (let-values ([(x y) (lt-find p p2)])
                                                                                (let-values ([(x2 y2) (lt-find p2 hole)])
                                                                                  (values (+ x x2) (+ y y2)))))
                                                              g1))])
                                      (let-values ([(x y) (cc-find p (dw-in chainD1-dw2))])
                                        (pin-over p x 0 incoming)))
                                    incoming
                                    (dw-in chainD1-dw2)
                                    'cw hole-etc.)
                 chain1-node1-code)))

    (define (chainD2-mid n)
      (chain chainD2-node1
             (redex (code (#,((if comp? comp cont) (chainD2-k-mid n)) #,v3)))))

    (define chainD2
      (chainD2-mid 1.0))
    
    (define chainD3-node3-expr (code (#,(v 15) #,hole)))
    
    (define chainD3-node3 (bubble chainD3-node3-expr))
    
    (define (mk-chainD3-dw3 suffix n3 n4 body4 alt-e4 n5)
      (let* ([e4 (or alt-e4 (e 4))]
             [b4-g (launder (ghost body4))]
             [p (dw-node** (- 1 n3)
                           n5
                           (code #,(e 3))
                           (cellophane
                            (if tagged?
                                (code #,p:dw #,(x 2))
                                (fade-pict n4
                                           (code #,(dw-inset (hbl-append p:dw suffix)))
                                           b4-g))
                            (- 1 n3))
                           e4)])
        (if (zero? n3)
            p
            (make-dw (pin-over (pin-over (dw-all p)
                                         e4 lt-find
                                         e4)
                               b4-g lt-find
                               (cellophane body4 n4))                     
                     (dw-in p) (dw-body p) (dw-out p)))))
    (define chainD3-dw3
      (mk-chainD3-dw3 (blank) 0 0 (blank) #f 0.0))
    (define (mk-chainD3-dw4 suffix n3 n4 body4 alt-e4 n5)
      (dw-node (code #,(e 5))
               (if tagged?
                   (code #,p:dw #,(x 3))
                   (code #,(dw-inset (hbl-append p:dw suffix))))
               (code #,(e 6))))
    (define chainD3-dw4
      (mk-chainD3-dw4 (blank) 0 0 (blank) #f 0.0))
    
    (define chainD3-redex-expr
      (code (k #,v3)))

    (define chainD3-redex 
      (redex chainD3-redex-expr))

    (define (make-chainD3-c1 chain1-node1 chainD1-dw2-all chain2-node2)
      (chain chain1-node1
             chainD1-dw2-all
             chain2-node2))
    (define (chainD3-c1* chain2-node2 tag)
      (let ([chainD1-dw2-all (if tag 
                                 (cc-superimpose
                                  (ghost (dw-all chainD1-dw2))
                                  (dw-all (make-chainD1-dw2 #f tag)))
                                 (dw-all chainD1-dw2))])
        (make-chainD3-c1 chain1-node1 chainD1-dw2-all chain2-node2)))
    (define chainD3-c1
      (chainD3-c1* chain2-node2 #f))
    (define chainD3-c2
      (chain (dw-all chainD3-dw3)
             chainD2-node1))
    (define (mk-chainD3-c3 n sub body suffix n3 n4 body4 bottom-tag alt-e4 n5)
      (chain (let-values ([(mk dw comb)
                           (if (= sub 15)
                               (values mk-chainD3-dw4 chainD3-dw4 lc-superimpose)
                               (values mk-chainD3-dw3 chainD3-dw3 rc-superimpose))])
               (let* ([p (dw-all dw)]
                      [p (if suffix
                             (let ([p2 (dw-all (mk suffix n3 n4 body4 alt-e4 n5))])
                               (refocus (comb (ghost p) p2)
                                        p2))
                             p)])
                 (refocus (cb-superimpose p bottom-tag)
                          p)))
             (- 1 n3)
             (bubble** (code (#,(v sub) #,body)) n)))

    (define chainD3-c3
      (pin-over (chain (dw-all chainD3-dw4)
                       chainD3-node3)
                chainD3-node3-expr ct-find
                ghost-hole-etc.))

    (define (add-D3-curve n midpt-list p start-pt)
      (apply curve-through-mid 
             n hole-etc. ghost-hole-etc.
             p
             (append
              (list start-pt
                    'ccw
                    (dw-out chainD3-dw3))
              midpt-list
              (list 'ccw
                    (dw-in chainD3-dw4)
                    'cw 
                    hole-etc.))))
    
    (define (mk-chainD3 n)
      (let ([c1 chainD3-c1]
            [c2 (chain chainD3-c2
                       chainD3-redex)]
            [c3 chainD3-c3])
        (add-D3-curve n
                      (list 'cw D3-midpt)
                      (pin-arrow-line
                       link-arrow-size
                       (pin-arrow-line link-arrow-size
                                       (vc-append (pict-height down-link)
                                                  c1
                                                  D3-midpt
                                                  (ht-append 20
                                                             c2
                                                             c3))
                                       c1 cb-find
                                       c2 ct-find)
                       c1 cb-find
                       c3 ct-find)
                      v3)))
    
    (define chainD3 (mk-chainD3 1.0))

    (define (make-common-chainD3-c1 chainD1-dw2)
      (if dfnt?
          (make-chainD3-c1 (refocus (outline-box (inset chain1-node1 2 0 2 2))
                                    chain1-node1)
                           (dw-all chainD1-dw2)
                           chain2-node2)
          (outline-box chainD3-c1)))
    
    (define common-chainD3-c1
      (make-common-chainD3-c1 chainD1-dw2-alt))
    
    (define chainD4-k
      (chain common-chainD3-c1 chainD3-c3))

    (define (mk-chainD4-k sub n body n2 body2 top-outline tag tag2 n3 n4 body4 bottom-tag alt-e4 n5)
      (chain (top-outline (chainD3-c1* (bubble** (code (#,body2 #,(v 2))) n2) tag))
             (- 1 n2)
             (mk-chainD3-c3 sub n body tag2 n3 n4 body4 bottom-tag alt-e4 n5)))

    (define chainD4-redex
      (redex (code (#,(cont (realign chainD4-k chain1-node1-code)) #,v3))))
    
    (define (specific a b)
      (if a
          (list a b)
          b))
    
    (define (chainD4-curve base dfnt? n d1-src d2-src)
      (add-D3-curve
       n
       (if dfnt?
           (list 'cw (specific d1-src (dw-out chainD1-dw2))
                 'cw D3-midpt
                 'ccw (specific d2-src (dw-in chainD1-dw2-alt)))
           (list 'cw D3-midpt))
       base
       D4-tag))
    
    (define chainD4
      (chainD4-curve
       (pin-arrow-line link-arrow-size
                       (ht-append 10
                                  (hc-append
                                   10
                                   (rb-superimpose
                                    (chain (make-common-chainD3-c1 chainD1-dw2)
                                           chainD3-c2)
                                    (inset D4-tag 0 0 5 0))
                                   (if dfnt?
                                       (inset D3-midpt 0 0 0 100)
                                       (inset D3-midpt 0 40 0 0)))
                                  chainD4-redex)
                       chainD2-node1 rc-find
                       chainD4-redex (lambda (p p2)
                                       (let-values ([(cx cy) (rc-find p chainD2-node1)]
                                                    [(rx ry) (lc-find p chainD4-redex)])
                                         (values rx cy))))
       dfnt?
       1.0
       #f #f))
    
    (values chainD1 chainD2 chainD2-mid chainD3 mk-chainD3 chainD4 chainD4-curve mk-chainD4-k
            chainD1+ chainD2-node1 chainD3-c1))
  
  (define-values (chainD1 chainD2 chainD2-mid chainD3 mk-chainD3 chainD4 chainD4-curve mk-chainD4-k
                          chainD1+ chainD2-node1 chainD3-c1)
    (make-chainDs #f #f #f))

  (define (chainD4-curve/fade n2 base dfnt? n d1-src d2-src)
    (if (zero? n2)
        (chainD4-curve base dfnt? n d1-src d2-src)
        (let* ([g1 (ghost base)]
               [p (chainD4-curve g1 dfnt? n d1-src d2-src)])
          (cc-superimpose base
                          (cellophane p (- 1 n2))))))
          
  (define (imagine p)
    (let ([col (make-object color% 200 200 200)])
      (cc-superimpose
       (colorize
        (rb-superimpose
         (lb-superimpose
          (cloud (pict-width p) (pict-height p) col)
          (filled-ellipse (/ (pict-width p) 2) (/ (pict-height p) 3)))
         (vl-append
          20
          (filled-ellipse 40 40)
          (inset
           (vl-append
            10
            (filled-ellipse 20 20)
            (inset (filled-ellipse 10 10) 20 0 0 0))
           40 0 0 0)))
        col)
       (scale p 0.85))))
  
  (define zero-chainD3 (mk-chainD3 0.0))
  (define full-chainD3 (mk-chainD3 1.0))
  
  (define (mk-chainD3/cache n)
    (cond
      [(zero? n) zero-chainD3]
      [(= n 1) full-chainD3]
      [else (mk-chainD3 n)]))
  
  ;; => A relatively POOR example
  ;; Too complicated, due to misguided re-use of abstractions set up
  ;;  for the paper's figures (see `make-chainDs').
  (define (dw-share-example-mid* copy-n eval1-n eval2-n eval3-n think? think-n orig-n 
                                 share-n revised-n tagged-n drop-n abort-n post-n postval-n done-n)
    (let* ([k (code k)]
           [kg1 (ghost k)]
           [kg2 (ghost k)]
           [kg3 (ghost k)]
           [kg4 (ghost k)]
           [kg5 (ghost k)]
           [kg6 (ghost k)]
           [share-box (lambda (p)
                        (refocus (cc-superimpose
                                  p
                                  (cellophane (outline-box (inset (ghost p) 10))
                                              share-n))
                                 p))]
           [tagged (lambda (v)
                     (fade-pict* lbl-superimpose 
                                 tagged-n 
                                 (blank) 
                                 (colorize
                                  (hbl-append (tt " ") (x v))
                                  dw-tag-color)))]
           [first-dw-tag (tagged 1)]
           [first-second-dw-tag (tagged 2)]
           [second-second-dw-tag (tagged 3)]
           [alt-e4 (e 4)]
           [alt-e4-g1 (ghost alt-e4)]
           [alt-e4-g2 (ghost alt-e4)]
           [bottom-tag (let ([p (blank (pict-width alt-e4) 0)])
                         (refocus (ct-superimpose 
                                   (cellophane down-link (* post-n
                                                            (- 1 done-n)))
                                   p)
                                  p))]
           [last-body (fade-pict eval2-n
                                 (code (#,p:call/cc (λ (k) #,kg1)))
                                 (code (#,kg6 #,v3)))]
           [last-body-g1 (ghost last-body)]
           [last-body-g2 (ghost last-body)]
           [last-body-g3 (ghost last-body)]
           [last-body-g4 (ghost last-body)]
           [chainD4-k (mk-chainD4-k (if (zero? eval3-n)
                                        eval1-n
                                        0.0)
                                    (if (zero? eval3-n)
                                        15
                                        6)
                                    (fade-pict eval2-n
                                               (fade-pict eval1-n hole kg2)
                                               hole)
                                    (if (zero? eval3-n)
                                        eval2-n
                                        (- 1 eval3-n))
                                    (fade-pict
                                     eval3-n
                                     (fade-pict eval2-n hole 
                                                (hbl-append
                                                 (code ...)
                                                 kg3
                                                 (code ...)))
                                     hole)
                                    share-box
                                    first-dw-tag
                                    first-second-dw-tag
                                    drop-n
                                    abort-n (fade-pict
                                             done-n
                                             (code (#,p:begin #,hole #,last-body-g3))
                                             last-body-g4)
                                    bottom-tag (if (zero? abort-n)
                                                   #f
                                                   alt-e4-g1)
                                    done-n)]
           [capturedD4-k (mk-chainD4-k 0.0 15
                                       (refocus (lt-superimpose hole
                                                                ghost-hole-etc.)
                                                hole)
                                       0.0 hole
                                       share-box
                                       first-dw-tag
                                       second-second-dw-tag
                                       0.0
                                       0.0 (blank)
                                       (blank) #f
                                       0.0)]
           [last-redex (cellophane
                        (vc-append
                         (cellophane down-link (- 1 drop-n))
                         (leads-to* postval-n
                                    (redex (fade-pict post-n
                                                      last-body-g1
                                                      alt-e4-g2))
                                    (v 4)))
                        (if (zero? done-n)
                            (if (zero? eval3-n)
                                (- 1 eval1-n)
                                eval3-n)
                            (- 1 done-n)))]
           [last-redex-g1 (launder (ghost last-redex))]
           [o (vc-append chainD4-k
                         last-redex-g1)]
           [o (slide-pict o last-redex last-redex-g1 bottom-tag post-n)]
           [o (if (zero? abort-n)
                  o
                  (slide-pict o last-body-g2 last-body-g3 last-body-g4 done-n))]
           [o (slide-pict o
                          (cellophane last-body (if (zero? eval1-n)
                                                    1.0
                                                    (if (zero? eval3-n)
                                                        (- 1 eval1-n)
                                                        eval3-n)))
                          last-body-g1 
                          (if (zero? abort-n) last-body-g1 last-body-g2)
                          abort-n)]
           [o (if (zero? abort-n)
                  o
                  (slide-pict o (cellophane alt-e4 (- 1 done-n))
                              alt-e4-g1 alt-e4-g2 post-n))])
      (ct-superimpose
       full-page
       (inset
        (scale 
         (let ([copied
                (let ([g1 (launder (ghost chainD4-k))])
                  (let ([together
                         (slide-pict (let ([o (cc-superimpose 
                                               o
                                               (blank (* 0.5 client-w) 0))])
                                       (pin-over (refocus
                                                  (ht-append (- 50)
                                                             (launder
                                                              ((if think? values ghost)
                                                               (inset (imagine
                                                                       (mk-chainD3/cache think-n))
                                                                      0 0 0 30)))
                                                             o)
                                                  o)
                                                 (+ 40 (pict-width o)) 0
                                                 g1))
                                     (cellophane (refocus (htl-append (code k)
                                                                      (tt " = ")
                                                                      capturedD4-k)
                                                          capturedD4-k)
                                                 copy-n)
                                     chainD4-k g1 (zoom copy-n))])
                    (chainD4-curve/fade drop-n
                                        (let* ([p (place-over together 
                                                              v3 find-cc
                                                              D4-tag)])
                                          (let-values ([(x1 y1) (cb-find p (list o chain1-node1))]
                                                       [(x2 y2) (cb-find p (list capturedD4-k chain1-node1))])
                                            (place-over p
                                                        (/ (+ x1 x2) 2) (if (zero? share-n)
                                                                            (/ (+ y1 y2) 2)
                                                                            (/ (pict-height p) 2))
                                                        D3-midpt)))
                                        (zero? share-n)
                                        (if (zero? share-n)
                                            orig-n
                                            revised-n)
                                        o capturedD4-k)))])
           (slide-pict
            (slide-pict
             (slide-pict copied
                         kg4 kg1 kg2 eval1-n)
             kg5 kg4 kg3 eval2-n)
            k kg5 kg6 eval3-n))
         0.9)
        0 gap-size 0 0))))
  
  (define (dw-share-example-mid . l)
    (apply dw-share-example-mid*
           (append l
                   (list-tail 
                    '(0.0 0.0 0.0 0.0 #f 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0)
                    (length l)))))
  
  (define (dw-share-copy-mid n)
    (dw-share-example-mid n))
  (define (dw-share-eval1-mid n)
    (dw-share-example-mid 1.0 n))
  (define (dw-share-eval2-mid n)
    (dw-share-example-mid 1.0 1.0 n))
  (define (dw-share-eval3-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 n))
  (define (dw-share-think-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t n))
  (define (dw-share-orig-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t 1.0 n))
  (define (dw-share-share-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t 1.0 1.0 n))
  (define (dw-share-revised-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t 1.0 1.0 1.0 n))
  (define (dw-share-tagged-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t 1.0 1.0 1.0 1.0 n))
  (define (dw-share-abort-drop-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t 1.0 1.0 1.0 1.0 1.0 n))
  (define (dw-share-abort-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t 1.0 1.0 1.0 1.0 1.0 1.0 n))
  (define (dw-share-post-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t 1.0 1.0 1.0 1.0 1.0 1.0 1.0 n))
  (define (dw-share-postval-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 n))
  (define (dw-share-done-mid n)
    (dw-share-example-mid 1.0 1.0 1.0 1.0 #t 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 n))
  
  
  (define chainD7-pre-e (e "pre"))
  (define chainD7-pre-last-e chainD7-pre-e)
  (define (chainD7-pre-last-v p) (leads-to p (v 1)))
  (define chainD7-post-e (e "post"))
  (define chainD7-body-e (e "body"))
  (define chainD7-body-v (v 2))
  (define chainD7-body-last-e chainD7-body-e)
  (define (chainD7-body-last-v p) (leads-to p chainD7-body-v))
  
  (define (squat amt p) (inset p 0 (* (pict-height p) 0.5 (- amt 1.0))))
  (define chainD7-node2
    (bubble (code (#,p:begin #,hole (#,p:dw #,(x 1) #,chainD7-pre-e #,chainD7-body-e #,chainD7-post-e)))))

  ;; => An OK example
  (define (dw-code-begin-example-mid dw-n n)
    (let* ([pre (code (grab-resource))]
           [body (code (work-thunk))]
           [post (code (release-with-resource))]
           
           [pre-g1 (ghost pre)]
           [pre-g2 (ghost pre)]
           [body-g1 (ghost body)]
           [body-g2 (ghost body)]
           [post-g1 (ghost post)]
           [post-g2 (ghost post)]
           
           [indent (lambda (p)
                     (hbl-append (tt " ") p))])
      (let* ([dw-expr
              (code (#,p:dynamic-wind
                     (λ () #,(fade-pict n 
                                        pre-g2
                                        chainD7-pre-e))
                     (λ () #,(fade-pict n
                                        body-g2
                                        chainD7-body-e))
                     (λ () #,(fade-pict n
                                        post-g2
                                        chainD7-post-e))))]
             [dw-expr-g1 (launder (ghost dw-expr))]
             [dw-expr-g2 (launder (ghost dw-expr))]
             
             [p
              (fade-pict 
               n
               (code (define (with-resource work-thunk)
                       #,(fade-pict
                          dw-n
                          (code (#,p:begin 
                                 #,(indent pre-g1)
                                 #,(indent body-g1)
                                 #,(indent post-g1)))
                          dw-expr-g1)))
               dw-expr-g2)]

             [p (slide-pict p (cellophane dw-expr dw-n) dw-expr-g1 dw-expr-g2 n)]
             [p (slide-pict p (cellophane pre (- 1 n)) pre-g1 pre-g2 dw-n)]
             [p (slide-pict p (cellophane body (- 1 n)) body-g1 body-g2 dw-n)]
             [p (slide-pict p (cellophane post (- 1 n)) post-g1 post-g2 dw-n)])
        p)))
  
  (define (dw-code-begin-mid n)
    (dw-code-begin-example-mid n 0.0))
  (define (dw-code-example-mid n)
    (dw-code-begin-example-mid 1.0 n))
  
  (define v3-ghost (ghost v3))  
  (define abort-expr
    (code (#,p:abort #,v3-ghost)))
  
  ;; => A slightly POOR example
  ;;  The nesting in the first expression bound to `p' is too deep.
  ;;  This experience led to longer `let*' chains to build up
  ;;  the picture in examples that were developed later (but some of
  ;;  which appear earlier in this file).
  (define (chainD7-generic-dw*-mid ex-n n predex? redex-n prebegin-n reduces-n predone-n spur-n 
                                   abort-n jump-n save-n post-n)
    (let* ([pre chainD7-pre-e]
           [post chainD7-post-e]
           [post-g7 (ghost post)]
           [abort-g1 (launder (ghost abort-expr))]
           [abort-g2 (launder (ghost abort-expr))]
           [body (evenize
                  (fade-pict abort-n 
                             chainD7-body-e
                             (fade-pict post-n
                                        (cellophane abort-g1
                                                    (- 1 save-n))
                                        post-g7)))])
      (let ([pre-g1 (ghost pre)]
            [pre-g2 (ghost pre)]
            [pre-g0 (ghost pre)]
            [pre-g3 (ghost pre)]
            [pre-g4 (ghost pre)]
            [body-g1 (ghost body)]
            [body-g2 (ghost body)]
            [body-g0 (ghost body)]
            [body-g3 (ghost body)]
            [post-g1 (ghost post)]
            [post-g2 (ghost post)]
            [post-g0 (ghost post)]
            [post-g3 (ghost post)]
            [post-g6 (ghost post)]
            [dw-g1 (ghost p:dw)]
            [dw-g2 (ghost p:dw)]
            [v3-g2 (ghost v3)])
        (let* ([example (code (#,p:dynamic-wind
                               (λ () #,pre-g1)
                               (λ () #,body-g1)
                               (λ () #,post-g1)))]
               [example-g1 (launder (ghost example))]
               [example-g2 (launder (ghost example))])
          (let* ([p (cc-superimpose
                     example-g1
                     (cellophane
                      (chain chain1-node1
                             (dw-node* (if (zero? jump-n)
                                           spur-n
                                           (- 1 jump-n))
                                       pre-g2 
                                       (bubble** (fade-pict
                                                  save-n
                                                  (fade-pict n
                                                             example-g2
                                                             (fade-pict
                                                              spur-n
                                                              (let* ([dw (code (#,dw-g1 #,pre-g3 #,body-g3 #,post-g3))]
                                                                     [dw+-g1 (launder (ghost dw))]
                                                                     [dw+-g2 (launder (ghost dw))])
                                                                (let ([b-n (if (zero? predone-n)
                                                                               prebegin-n
                                                                               (- 1 predone-n))])
                                                                  (slide-pict
                                                                   (fade-pict
                                                                    b-n
                                                                    dw+-g1
                                                                    (code (#,p:begin #,hole #,dw+-g2)))
                                                                   dw dw+-g1 dw+-g2 b-n)))
                                                              (dw-inset dw-g2)))
                                                  (code (#,p:begin #,hole #,abort-g2)))
                                                 (if (zero? predone-n)
                                                     (- 1.0 redex-n)
                                                     predone-n))
                                       post-g2)
                             (if (zero? predone-n)
                                 redex-n
                                 (- 1 predone-n))
                             (leads-to* (if (zero? predone-n) 
                                            reduces-n 
                                            (- 1 predone-n))
                                        (redex (cbl-superimpose
                                                pre-g4
                                                body-g2))
                                        (v 1)))
                      ex-n))]
                 [p (slide-pict p 
                                (cellophane p:dw (if (zero? jump-n)
                                                     n
                                                     (- 1 jump-n)))
                                dw-g1 dw-g2 (zoom spur-n))]
                 [p (slide-pict p (cellophane example (- 1.0 n)) example-g1 example-g2 (zoom ex-n))]
                 [p (slide-pict p body-g0 body-g1 body-g3 (zoom n))]
                 [p (slide-pict p pre-g0 pre-g1 pre-g3 (zoom n))]
                 [p (slide-pict p post-g0 post-g1 post-g3 (zoom n))]
                 [p (slide-pict p body body-g0 body-g2 (zoom (if predex? 0 redex-n)))]
                 [p (slide-pict p (cellophane pre (- 1 jump-n)) pre-g0 pre-g2 (zoom spur-n))]
                 [p (if predex?
                        (slide-pict p (cellophane pre (- 1 predone-n)) pre-g0 pre-g4 (zoom redex-n))
                        p)]
                 [p (slide-pict p post-g6 post-g0 post-g2 (zoom spur-n))]
                 [p (slide-pict p post post-g6 post-g7 (zoom post-n))]
                 [p (slide-pict p (cellophane abort-expr abort-n) abort-g1 abort-g2 (zoom save-n))]
                 [p (slide-pict p (cellophane v3 abort-n) v3-ghost v3-ghost (zoom save-n))])
            p)))))
  
  (define (chainD7-generic-dw-mid n)
    (chainD7-generic-dw*-mid 1.0 n #f 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0))

  (define (chainD7-generic-redex-mid n)
    (chainD7-generic-dw*-mid 1.0 1.0 #f n 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0))
  
  (define (chainD7-generic-predex-mid n)
    (chainD7-generic-dw*-mid 1.0 1.0 #t n 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0))

  (define (chainD7-generic-prebegin-mid n)
    (chainD7-generic-dw*-mid 1.0 1.0 #t 1.0 n 0.0 0.0 0.0 0.0 0.0 0.0 0.0))

  (define (chainD7-generic-prereduces-mid n)
    (chainD7-generic-dw*-mid 1.0 1.0 #t 1.0 1.0 n 0.0 0.0 0.0 0.0 0.0 0.0))

  (define (chainD7-generic-predone-mid n)
    (chainD7-generic-dw*-mid 1.0 1.0 #t 1.0 1.0 1.0 n 0.0 0.0 0.0 0.0 0.0))

  (define (chainD7-generic-spurs-mid n)
    (chainD7-generic-dw*-mid 1.0 1.0 #f 1.0 0.0 0.0 0.0 n 0.0 0.0 0.0 0.0))

  (define (chainD7-example-to-generic-mid n)
    (if (zero? n)
        (dw-code-example-mid 1.0)
        (chainD7-generic-dw*-mid n 0.0 #f 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0)))
  
  (define (chainD7-generic-to-abort-mid n)
    (chainD7-generic-dw*-mid 1.0 1.0 #f 1.0 1.0 0.0 0.0 1.0 n 0.0 0.0 0.0))
  
  (define (pin-above base over p2)
    (let-values ([(x y) (cc-find base over)])
      (pin-over base
                (- x (/ (pict-width p2) 2)) 0
                p2)))
  
  (define chainD7-exit-mid
    (let ([base (chainD7-generic-to-abort-mid 1.0)])
      (lambda (n)
        (chainD7-abort-curve n base))))
  
  (define (chainD7-abort-curve n base)
    (let ([g1 (ghost etc.)])
      (evenize
       (curve-through-mid n etc. g1
                          (pin-above base
                                     chainD7-post-e
                                     g1)
                          abort-expr
                          chainD7-post-e
                          etc.))))

  (define chainD7-add-jump-in-mid-mid
    (let ([jump-out (chainD7-exit-mid 1.0)])
      (lambda (n in-n)
        (let ([g1 (ghost jump-out)]
              [g2 (ghost jump-out)])
          (slide-pict (cc-superimpose
                       (evenize
                        (hc-append 50
                                   g2
                                   (cellophane (chainD2-mid in-n) n)))
                       g1)
                      jump-out g1 g2 (zoom n))))))
  
  (define (chainD7-add-jump-in-mid n)
    (chainD7-add-jump-in-mid-mid n 0.0))

  (define (chainD7-jump-in-mid n)
    (chainD7-add-jump-in-mid-mid 1.0 n))
  
  (define (chainD7-remove-jump-in-mid n)
    (chainD7-add-jump-in-mid-mid (- 1 n) 1.0))

  (define (chainD7-start-abort-mid n)
    (let ([p (chainD7-generic-dw*-mid 1.0 1.0 #f 1.0 1.0 0.0 0.0 1.0 1.0 n 0.0 0.0)])
      (cc-superimpose p
                      (cellophane (chainD7-abort-curve 1.0 (ghost p)) (- 1 n)))))

  (define (chainD7-abort-save-mid n)
    (chainD7-generic-dw*-mid 1.0 1.0 #f 1.0 1.0 0.0 0.0 1.0 1.0 1.0 n 0.0))

  (define (chainD7-abort-post-mid n)
    (chainD7-generic-dw*-mid 1.0 1.0 #f 1.0 1.0 0.0 0.0 1.0 1.0 1.0 1.0 n))

  ;; ----------------------------------------
  
  (define exn-impl
    (code
     (code:line
      (#,p:handle #,(e 1) (λ (x) #,(e 2))) 
      code:blank = code:blank (#,p:call/cm 
                               #,p:handle-mark-key (λ (x) #,(e 2))
                               (λ () #,(e 1)))
      code:blank
      (#,p:catch #,(e 1) (λ (x) #,(e 2)))
      code:blank = code:blank (#,p:prompt #,(code
                                             (code:line
                                              #,p:catch-tag 
                                              (#,p:handle #,(e 1) (λ (x) (#,p:abort  #,p:catch-tag  x)))
                                              (λ (x) #,(e 2)))))
      code:blank
      (#,p:throw #,(v 1))
      code:blank = code:blank (#,p:default-exn-handler
                               (#,p:fold #,(code
                                            (code:line
                                             (λ (v h) (h v))
                                             #,(v 1)
                                             (#,p:current-marks   #,p:handle-mark-key))))))))
  
  ;; ----------------------------------------

  (define drscheme-excerpt
    (parameterize ([current-code-font 'modern])
      (code
       (code:line
        (define (evaluate-from-port port complete-program?)
          (define tag (default-continuation-prompt-tag))
          (define (loop)
            (let ([expr (get-next-expression port)])
              (unless (eof-object? expr)
                (code:comment "Delimit each top-level evaluation:")
                (call-with-continuation-prompt
                 (λ () (eval expr)) tag
                 (if complete-program?
                     (code:comment "Load mode: don't continue")
                     (λ args
                        (abort-current-continuation 
                         tag void))
                     (code:comment "REPL mode: run thunk and continue")
                     (λ (thunk)
                        (call-with-continuation-prompt 
                         thunk tag))))
                (loop))))
          (code:comment "Catch possible escape in load mode:")
          (call-with-continuation-prompt loop))))))

  ;; ----------------------------------------

  (define (get-prog sub)
    (code (let ([terms #,sub])
            (find-paper terms))))
  
  (define pick-flight-expr
    (code (get-search-terms)))
  
  (define search-terms-expr
    (code '("control"
            "production"
            "environment")))
  
  (define serve-expr
    (code (#,s:serve #,((mv "out") 1) #,((mv "in") 1))))
  
  ;; => A relatively GOOD example
  ;;  This one is complicated, but driving a function like this
  ;;  with `play-n' works well. Abstracting over manual creation
  ;;  of ghosts and slides might be one way to write these
  ;;  animations better.
  (define (web-server-mid serve-n ;; expand serve call
                          dispatch-n ;; call/cc sub-expr
                          inscc-n ;; insert call/cc around dispatch
                          copy-n ;; save esc
                          catch-n ;; dispatch sub-expr
                          bind-n ;; let sub-expr
                          plan-n ;; bind sub-expr
                          ;; get-n ;; to general do-form reply
                          encode-n ;; to inner call/cc
                          save-copy-n ;; save session
                          save-n ;; cont saved
                          form-n ;; apply html-generator
                          ; esc-n ;; move continuation into app position
                          ; drop-n ;; drop continuation
                          ; restore-n ;; restore old
                          finish-n ;; HTML to reply
                          next-n ;; next-time
                          k-n ;; dispatch to k
                          krator-n ;; k into rator position
                          dot-n ;; hilite wrong output
                          )
    (let* ([lg (lambda (p) (launder (ghost p)))]
           
           [esc-n 0.0]
           [drop-n finish-n]
           [restore-n (if (zero? finish-n)
                          0
                          1.0)]

           [dispatch (fade-pict inscc-n
                                s:generate-html
                                s:dispatch)]
           [mk-dispatch-expr (lambda (n)
                               (code (#,dispatch #,((mv "in") n))))]
           [dispatch-expr (mk-dispatch-expr 1)]
           [dispatch-expr-g1 (lg dispatch-expr)]
           [dispatch-expr-g2 (lg dispatch-expr)]
           [dispatch-expr-g5 (lg dispatch-expr)]
           [dispatch-expr-g6 (lg dispatch-expr)]
           [dispatch-expr-g7 (lg dispatch-expr)]
           [dispatch-expr-g8 (lg dispatch-expr)]
           
           [cc-esc-expr (code (#,p:call/cc
                                 (λ (esc) #,dispatch-expr-g6)))]
           [cc-esc-expr-g1 (lg cc-esc-expr)]
           [cc-esc-expr-g2 (lg cc-esc-expr)]
            
           [html (cellophane (t "<html/>") (- 1 next-n))]
           [html-g1 (lg html)]
           [html-g2 (lg html)]
           
           [mk-top*
            (lambda (body n)
              (let ([out ((mv "out") n)])
                (cons (bubble** (fade-pict
                                 serve-n
                                 serve-expr
                                 (code (#,s:reply #,out #,body)))
                                (- 1 dispatch-n))
                      out)))]
           [mk-top (lambda (body n)
                     (car (mk-top* body n)))]
           
           [top+out (mk-top* (fade-pict
                              dispatch-n
                              dispatch-expr-g7
                              hole)
                             1)]
           [top (car top+out)]
           [out (cdr top+out)]
           
           [restored-top (mk-top (fade-pict
                                  finish-n
                                  hole
                                  html-g2)
                                 1)]
           
           [top-g2 (lg top)]
           [top-g3 (lg top)]
           
           [get-expr pick-flight-expr]
           [get-expr-g1 (lg get-expr)]
           [get-expr-g2 (lg get-expr)]
           
           [mid
            (bubble** (fade-pict
                       inscc-n
                       dispatch-expr-g8
                       (fade-pict
                        catch-n
                        cc-esc-expr-g2
                        (fade-pict
                         bind-n
                         dispatch-expr-g2
                         (get-prog (fade-pict
                                    plan-n
                                    get-expr-g1
                                    hole)))))
                      (- 1 (* 0.5 plan-n)))]
           
           [k (chain top
                     dispatch-n
                     mid)]
           
           [mk-html-expr
            (code build-form
                  #;
                  (λ (k-url)
                     (html ...)))]
           [mk-html-expr-g1 (lg mk-html-expr)]
           [mk-html-expr-g2 (lg mk-html-expr)]
           
           [esc-rator (fade-pict
                       esc-n
                       (code esc)
                       top-g2)]
            
           [esc-expr
            (code (#,esc-rator #,(fade-pict
                                  form-n
                                  (code (#,mk-html-expr-g2 (#,s:cont->url web-k)))
                                  html-g1)))]
           [esc-expr-g1 (lg esc-expr)]
           [esc-expr-g2 (lg esc-expr)]

           [bottom (redex 
                    (fade-pict
                     encode-n ;; get-n
                     get-expr-g2
                     (fade-pict
                      1.0 ;; encode-n
                      (code (#,s:do-form #,mk-html-expr-g1))
                      (fade-pict
                       save-n
                       (code (#,p:call/cc
                              (λ (web-k)
                                 #,esc-expr-g1)))
                       esc-expr-g2))))]
           [bottom-g1 (lg bottom)]
           
           [p (cellophane (chain k
                                 plan-n
                                 bottom-g1)
                          (- 1 drop-n))]
           
           [p (slide-pict p (cellophane bottom (* plan-n (- 1 finish-n)))
                          bottom-g1 bottom-g1 0.0)]
           
           [p (cellophane p (- 1 next-n))]
           
           [p (slide-pict p (cellophane cc-esc-expr (* inscc-n (- 1 catch-n)))
                          cc-esc-expr-g1 cc-esc-expr-g2 dispatch-n)]
           [p (slide-pict p dispatch-expr-g5 
                          dispatch-expr-g7 dispatch-expr-g8 dispatch-n)]
           [p (slide-pict p dispatch-expr-g1 
                          dispatch-expr-g5 dispatch-expr-g6 inscc-n)]
           [p (slide-pict p (cellophane dispatch-expr (* serve-n (- 1 bind-n)))
                          dispatch-expr-g1 dispatch-expr-g2 catch-n)]
           [p (slide-pict p (cellophane get-expr (* bind-n (- 1 encode-n)))
                          get-expr-g1 get-expr-g2 plan-n)]
           [p (slide-pict p (cellophane esc-expr (* encode-n (- 1 finish-n)))
                          esc-expr-g1 esc-expr-g2 save-n)]
           [p (slide-pict p (cellophane mk-html-expr (* #|get-n|# encode-n (- 1 form-n)))
                          mk-html-expr-g1 mk-html-expr-g2 1.0 #|encode-n|#)]
           
           [small 0.5]
           
           [small-top-g2 (lg (scale top small))]
           
           [p (ct-superimpose (inset p 0 gap-size 0 0)
                              titleless-page)]
           
           [p (lt-superimpose p
                              (cellophane
                               (htl-append (scale (code esc) small)
                                           (scale (tt " = ") small)
                                           small-top-g2)
                               copy-n))]

           [small-k-g2 (lg (scale k small))]

           [p (rt-superimpose p
                              (cellophane
                               (ltl-superimpose
                                (let ([p (scale (tt " = ") small)])
                                  (refocus (htl-append (scale (code web-k) small)
                                                       p)
                                           p))
                                small-k-g2)
                               save-copy-n))]

           [p (slide-pict p (launder
                             (cellophane (scale top (+ small (* (- 1 small) (- 1 copy-n))))
                                         copy-n))
                          top small-top-g2 copy-n)]
           [p (slide-pict p (launder
                             (cellophane (scale k (+ small (* (- 1 small) (- 1 save-copy-n))))
                                         save-copy-n))
                          k small-k-g2 save-copy-n)]
           
           [p (slide-pict p top-g3
                          small-top-g2 top-g2 esc-n)]
           [p (slide-pict p (let ([t (cellophane (scale restored-top 
                                                        (if (zero? restore-n)
                                                            (+ small (* (- 1 small) esc-n))
                                                            1.0))
                                                 (* (if (zero? restore-n)
                                                        esc-n
                                                        1.0)
                                                    (- 1 next-n)))])
                              (inset t (/ (- (pict-width top) (pict-width t)) 2) 0))
                          top-g3 top restore-n)]
           
           [p (slide-pict p (cellophane html form-n)
                          html-g1 html-g2 finish-n)]
           
           [k-g3 (lg k)]
           [k-rator (fade-pict krator-n
                               (code web-k)
                               k-g3)]
           
           [p2 (inset (cellophane (chain (mk-top hole 2)
                                         (redex (fade-pict
                                                 k-n
                                                 (mk-dispatch-expr 2)
                                                 (code (#,k-rator form-answers)))))
                                  next-n)
                      0 gap-size 0 0)]
           
           [p (ct-superimpose p p2)]
           
           [p (slide-pict p (cellophane (let ([p (scale k (+ small (* (- 1 small) krator-n)))])
                                          (pin-over p
                                                    out lt-find
                                                    (hilite-dot dot-n out)))
                                        krator-n)
                          small-k-g2 k-g3 krator-n)]
           
           )
      p))
  
  ;; => A relatively GOOD example
  ;;  This one looks a lot like web-sever-mid'. Abstracting over
  ;;  `web-server-mid', though, would have created a tangled
  ;;  mess. Instead, a few shared pieces have been lifted out, and
  ;;  the two `web-server' functions assemble them in slightly
  ;;  different ways.
  (define ((web-server-delim-mid alt?) prompted-n
                                       body-n
                                       copy-n
                                       toabort-n
                                       abort-n
                                       second-n
                                       callk-n
                                       restore-n
                                       compose-n
                                       finish-n)
    (let* ([lg (lambda (p) (launder (ghost p)))]
           
           [html (t "<html/>")]
           [html-g1 (lg html)]
           [html-g2 (lg html)]
           
           [prompt-n 1.0]

           [mk-dispatch-expr (lambda (n)
                               (code (#,s:dispatch #,((mv "in") n))))]
           [dispatch-expr (mk-dispatch-expr 1)]
           [dispatch-expr-g1 (lg dispatch-expr)]
           [dispatch-expr-g2 (lg dispatch-expr)]
           
           [prompt-expr (code (#,p:prompt #,(fade-pict
                                             prompted-n
                                             dispatch-expr-g1
                                             hole)))]
           [prompt-expr-g1 (lg prompt-expr)]
           [prompt-expr-g2 (lg prompt-expr)]
           
           [top
            (bubble** (code (#,s:reply #,((mv "out") 1) #,(fade-pict
                                                           abort-n
                                                           (fade-pict
                                                            prompt-n
                                                            prompt-expr-g1
                                                            hole)
                                                           html-g2)))
                      (- 1 prompt-n))]
           
           [prompt-node (bubble** prompt-expr-g2
                                  (- 1 prompt-n))]
           
           [mk-body-node (lambda (shape-n hole)
                           (bubble** (fade-pict
                                      body-n
                                      dispatch-expr-g2
                                      (get-prog hole))
                                shape-n))]
           [body-node (mk-body-node (- 1 body-n) hole)]
           
           [abort-expr (code (#,p:abort #,(fade-pict
                                           toabort-n
                                           (code ...)
                                           html-g1)))]
           [abort-expr-g1 (lg abort-expr)]
           [abort-expr-g2 (lg abort-expr)]
           
           [p (chain top
                     (* prompt-n (- 1 abort-n))
                     prompt-node
                     prompted-n
                     body-node
                     body-n
                     (redex (fade-pict
                             toabort-n
                             (code (#,p:call/cc
                                    (λ (web-k)
                                       #,abort-expr-g1)))
                             abort-expr-g2)))]
           
           [p (ct-superimpose (inset p 0 gap-size 0 0)
                              titleless-page)]
           
           [p (slide-pict p (cellophane prompt-expr (- 1 abort-n))
                          prompt-expr-g1 prompt-expr-g2 prompt-n)]
           [p (slide-pict p (cellophane dispatch-expr (- 1 body-n))
                          dispatch-expr-g1 dispatch-expr-g2 prompted-n)]
           [p (slide-pict p (cellophane abort-expr (* body-n (- 1 abort-n)))
                          abort-expr-g1 abort-expr-g2 toabort-n)]
           [p (slide-pict p (cellophane html toabort-n)
                          html-g1 html-g2 abort-n)]
           
           [p (cellophane p (- 1 second-n))]
           
           [form-vals (if alt?
                          search-terms-expr
                          (code form-answers))]
           [form-vals-g1 (lg form-vals)]
           [form-vals-g2 (lg form-vals)]
           
           [body-node2 (mk-body-node finish-n
                                     (fade-pict
                                      finish-n
                                      hole
                                      form-vals-g2))]
           [k-g2 (lg body-node)]
           [k-g3 (lg body-node)]
           [k-g4 (lg body-node)]
           [k-rator (fade-pict
                     restore-n
                     (code web-k)
                     (fade-pict compose-n
                                k-g2
                                (ghost hole)))]
           
           [p (ct-superimpose p
                              (cellophane
                               (inset
                                (chain (bubble (code (#,s:reply #,((mv "out") 2) #,hole)))
                                       (vc-append
                                        (bubble (code (#,p:prompt #,hole)))
                                        (fade-pict* ct-superimpose
                                                    compose-n
                                                    (blank)
                                                    down-link)
                                        (fade-pict compose-n
                                                   (blank)
                                                   k-g3))
                                       (- 1 finish-n)
                                       (redex (fade-pict
                                               callk-n
                                               (mk-dispatch-expr 2)
                                               (code (#,k-rator #,form-vals-g1)))))
                                0 gap-size 0 0)
                               second-n))]
           
           [small 0.5]
           [small-k-g2 (scale (launder (ghost body-node)) small)]

           [p (rt-superimpose p
                              (inset
                               (cellophane
                                (htl-append (scale (code web-k) small)
                                            (scale (tt " = ") small)
                                            small-k-g2)
                                copy-n)
                               0 80 0 0))]
           
           [p (slide-pict p k-g4
                          small-k-g2 k-g2 restore-n)]           
           [p (slide-pict p (cellophane (scale body-node2 (+ small (* (- 1 small) restore-n)))
                                        restore-n)
                          k-g4 k-g3 compose-n)]
           [p (slide-pict p (cellophane form-vals callk-n)
                          form-vals-g1 form-vals-g2 finish-n)]
                              
           [p (slide-pict p (cellophane (scale body-node (+ small (* (- 1 small) (- 1 copy-n))))
                                        copy-n)
                          body-node small-k-g2 copy-n)]
           )
      p))
  
  (void))
